import configvalues as conf
from CalLabel import CalLabel

import gtk
import pango
import commands
import time
import os



_MONTHS = [_("January"), _("February"), _("March"),
           _("April"), _("May"), _("June"),
           _("July"), _("August"), _("September"),
           _("October"), _("November"), _("December")]

_WEEKDAYS = [_("Mo"), _("Tu"), _("We"), _("Th"), _("Fr"), _("Sa"), _("Su")]



# TODO: get rid of 'cal' dependency
_CAL_CMD = "sh -c 'LANGUAGE=C LC_ALL=C LANG=C cal -1 -m %(month)d %(year)d'"
_CAL_CMD_FALLBACK = "sh -c 'LANGUAGE=C LC_ALL=C LANG=C cal %(month)d %(year)d'"



#
# Class for a calendar widget.
#
class Cal(gtk.Fixed):

    #
    # Constructor.
    #
    def __init__(self):

        self.__month = 1
        self.__year = 1

        self.__cal_cmd = _CAL_CMD

        self.__have_today_mark = 0

        
        gtk.Fixed.__init__(self)

        # month and year
        self.__lbl_month = CalLabel(2, 1, gtk.FALSE)
        self.__lbl_month.show()
        self.put(self.__lbl_month, 0, 0)

        # weekdays
        self.__lbl_weekdays = CalLabel(7, 1, gtk.TRUE)
        self.__lbl_weekdays.show()
        self.put(self.__lbl_weekdays, 0, 0)

        # days
        self.__lbl_days = CalLabel(7, 6, gtk.TRUE)
        self.__lbl_days.show()
        self.put(self.__lbl_days, 0, 0)

        # mark for today
        self.__today_mark = gtk.Image()
        self.__today_mark.show()
        self.put(self.__today_mark, 0, 0)
        


    #
    # Places the labels.
    #
    def __place_labels(self, config):

        # place month/year
        xpos_month = config.get([conf.SKIN, conf.GEOMETRY, conf.MONTH, conf.X])
        ypos_month = config.get([conf.SKIN, conf.GEOMETRY, conf.MONTH, conf.Y])

        self.move(self.__lbl_month, xpos_month, ypos_month)


        # place weekdays
        xpos_wdays = config.get([conf.SKIN, conf.GEOMETRY, conf.WEEKDAYS,
                                     conf.X])
        ypos_wdays = config.get([conf.SKIN, conf.GEOMETRY, conf.WEEKDAYS,
                                     conf.Y])

        self.move(self.__lbl_weekdays, xpos_wdays, ypos_wdays)


        # place days
        xpos_days = config.get([conf.SKIN, conf.GEOMETRY, conf.DAYS, conf.X])
        ypos_days = config.get([conf.SKIN, conf.GEOMETRY, conf.DAYS, conf.Y])

        self.move(self.__lbl_days, xpos_days, ypos_days)
        




    #
    # Reacts on entering a day label.
    #
    def __on_day_enter(self, src, event, lbl):

        # TODO: reserved for future versions
        
        text = lbl.get_text().strip()
        if (not text.isdigit()): return

        day = int(text)
        month = self.__month
        year = self.__year




    #
    # Reacts on leaving a day label.
    #
    def __on_day_leave(self, src, event, lbl):

        pass




    #
    # Returns the current day, month and year.
    #
    def __get_current_day(self):

        now = time.localtime(time.time())
        year = now[0]
        month = now[1]
        day = now[2]

        return (day, month, year)




    #
    # Sets the month to be displayed.
    #
    def set_month(self, month = 0, year = 0):

        curday, curmonth, curyear = self.__get_current_day()
        if (month == 0 or year == 0):
            month = curmonth; year = curyear

        self.__month = month
        self.__year = year

        # get the calendar sheet
        fail, cal = commands.getstatusoutput(self.__cal_cmd % vars())
        if (fail):
            self.__cal_cmd = _CAL_CMD_FALLBACK
            fail, cal = commands.getstatusoutput(self.__cal_cmd % vars())
            
        lines = cal.splitlines()

        monthtext, yeartext = lines[0].split()[:2]
        days = lines[1].strip() + "\n" + "\n".join(lines[2:])

        lbl = self.__lbl_month.get_label(0)
        lbl.set_text(_(monthtext) + " ")

        lbl = self.__lbl_month.get_label(1)
        lbl.set_text("\n" + yeartext + " ")

        # update labels
        self.__today_mark.hide()

        for i in xrange(7):
            value = days[i * 3:i * 3 + 2]
            lbl = self.__lbl_weekdays.get_label(i)
            lbl.set_text(_(value))


        for i in xrange(42):
            value = days[(i + 7) * 3:(i + 7) * 3 + 2]
            lbl = self.__lbl_days.get_label(i)
            lbl.set_text(value)

            if (value.strip().isdigit()):
                day = int(value.strip())
            else:
                day = 0

            # place the mark
            if (self.__have_today_mark and
                month == curmonth and year == curyear and day == curday):
                alloc = lbl.get_allocation()
                mark_alloc = self.__today_mark.get_allocation()
                self.move(self.__today_mark,
                          alloc.x + (alloc.width - mark_alloc.width) / 2,
                          alloc.y + (alloc.height - mark_alloc.height) / 2)
                self.__today_mark.show()
            #end if
        #end for




    #
    # Returns the currently displayed month and year.
    #
    def get_month(self):

        return (self.__month, self.__year)



    #
    # Scales the given font description from point size to pixel size.
    #
    def __scale_font(self, fontdescription, dpi):

        height = gtk.gdk.screen_height()
        height_inch = height / dpi
        height_pt = height_inch * 72.0
        scale = height_pt / height

        size = fontdescription.get_size()
        size *= scale
        
        fontdescription.set_size(size)



    #
    # Configures the calendar.
    #
    def set_config(self, config):

        # resolution
        dpi = float(config.get([conf.TEMPORARY, conf.DPI]))

        # fonts
        font_year = pango.FontDescription(
            config.get([conf.SKIN, conf.FONTS, conf.YEAR]))
        font_month = pango.FontDescription(
            config.get([conf.SKIN, conf.FONTS, conf.MONTH]))
        font_weekdays = pango.FontDescription(
            config.get([conf.SKIN, conf.FONTS, conf.WEEKDAYS]))
        font_days = pango.FontDescription(
            config.get([conf.SKIN, conf.FONTS, conf.DAYS]))

        # colors
        color_month = config.get([conf.SKIN, conf.COLORS, conf.MONTH])
        color_year = config.get([conf.SKIN, conf.COLORS, conf.YEAR])
        color_weekdays = config.get([conf.SKIN, conf.COLORS, conf.WEEKDAYS])
        color_days = config.get([conf.SKIN, conf.COLORS, conf.DAYS])


        # mark for today
        path = config.get([conf.SKIN, conf.PATH])
        today_mark = config.get([conf.SKIN, conf.GRAPHICS, conf.TODAY])
        if (today_mark):
            self.__today_mark.set_from_file(os.path.join(path, today_mark))
            self.__have_today_mark = 1
        else:
            self.__have_today_mark = 0

        
        self.__scale_font(font_month, dpi)
        self.__scale_font(font_year, dpi)
        self.__scale_font(font_weekdays, dpi)
        self.__scale_font(font_days, dpi)
        

        # geometry
        width_wdays = config.get([conf.SKIN, conf.GEOMETRY, conf.WEEKDAYS,
                                      conf.WIDTH])
        height_wdays = config.get([conf.SKIN, conf.GEOMETRY, conf.WEEKDAYS,
                                       conf.HEIGHT])
        hspace_wdays = config.get([conf.SKIN, conf.GEOMETRY, conf.WEEKDAYS,
                                       conf.HSPACING])

        width_days = config.get([conf.SKIN, conf.GEOMETRY, conf.DAYS,
                                 conf.WIDTH])
        height_days = config.get([conf.SKIN, conf.GEOMETRY, conf.DAYS,
                                  conf.HEIGHT])
        hspace_days = config.get([conf.SKIN, conf.GEOMETRY, conf.DAYS,
                                  conf.HSPACING])
        vspace_days = config.get([conf.SKIN, conf.GEOMETRY, conf.DAYS,
                                  conf.VSPACING])

        # month and year
        lbl = self.__lbl_month.get_label(0)
        lbl.modify_font(font_month)
        lbl.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse(color_month))

        lbl = self.__lbl_month.get_label(1)
        lbl.modify_font(font_year)
        lbl.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse(color_year))


        # weekdays
        self.__lbl_weekdays.set_spacing(hspace_wdays, 0)
        for i in xrange(7):
            lbl = self.__lbl_weekdays.get_label(i)
            lbl.set_size_request(width_wdays, height_wdays)
            lbl.modify_font(font_weekdays)
            lbl.modify_fg(gtk.STATE_NORMAL,
                          gtk.gdk.color_parse(color_weekdays))

        # days
        self.__lbl_days.set_spacing(hspace_days, vspace_days)
        for i in xrange(42):
            lbl = self.__lbl_days.get_label(i)
            lbl.set_size_request(width_days, height_days)
            lbl.modify_font(font_days)
            lbl.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse(color_days))

        self.__place_labels(config)


