from Observable import Observable
import values

import gtk
import gobject

import os



#
# Class for a browser for skins.
#
class SkinBrowser(gtk.Dialog, Observable):

    OBS_SET_SKIN = 0



    #
    # Constructor.
    #
    def __init__(self):

        # the list of available skins
        self.__skins = []


        # widgets and layout
        gtk.Dialog.__init__(self)
        self.set_title(_("Skin Browser"))
        self.set_default_size(320, 240)
        self.connect("delete-event", self.__on_close)

        scrwin = gtk.ScrolledWindow()
        scrwin.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        scrwin.set_policy(gtk.POLICY_NEVER,
                          gtk.POLICY_AUTOMATIC)
        scrwin.show()
        self.vbox.pack_start(scrwin, gtk.TRUE, gtk.TRUE, 0)

        self.__liststore = gtk.ListStore(gobject.TYPE_STRING)
        self.__list = gtk.TreeView(self.__liststore)
        self.__list.append_column(gtk.TreeViewColumn(_("Name of Skin"),
                                                     gtk.CellRendererText(),
                                                     text=0))
        self.__list.show()

        self.__list.connect("button-release-event", self.__on_select)
        scrwin.add(self.__list)

        btn = self.add_button(gtk.STOCK_CLOSE, 0)
        btn.connect("clicked", self.__on_close)


        self.__display_skins()




    #
    # Displays the available skins in the list view.
    #
    def __display_skins(self):

        self.__liststore.clear()
        self.__skins = []
        skins = self.__get_installed_skins()
        skins.sort()
        
        for skin in skins:
            self.__skins.append(skin)
            iter = self.__liststore.append()
            name = skin.replace("_", " ")
            self.__liststore.set(iter, 0, name)
        #end for




    #
    # Reacts on selecting a skin.
    #
    def __on_select(self, src, event):

        index = src.get_cursor()[0][0]
        skin = self.__skins[index]
        button = event.button
        if (button == 1):
            self.update_observer(self.OBS_SET_SKIN, skin)



    #
    # Reacts on closing the window.
    #
    def __on_close(self, *args):

        self.hide()
        return gtk.TRUE




    #
    # Finds and returns the names of all installed skins.
    #
    def __get_installed_skins(self):

        home = os.environ[values.DESKCALHOME]
        userhome = os.environ[values.DESKCALUSERHOME]

        skins = {}
        for path in [os.path.join(home, "skins"),
                     os.path.join(userhome, "skins")]:
            dirs = os.listdir(path)

            for d in dirs:
                if (os.path.exists(os.path.join(path, d, "skin.xml"))):
                    skins[d] = 1
            #end for
            
        #end for
        
        return skins.keys()



    #
    # Shows the window.
    #
    def show(self):

        self.__display_skins()
        gtk.Dialog.show(self)
        



    #
    # Marks the given skin as selected.
    #
    def set_current_skin(self, skin):

        try:
            index = self.__skins.index(skin)
            self.__list.set_cursor(index, None, gtk.FALSE)
        except:
            pass
